package com.socialproximity.socprox;

import static com.socialproximity.socprox.CommonUtilities.SENDER_ID;
import static com.socialproximity.socprox.CommonUtilities.displayMessage;

import org.json.JSONObject;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import com.google.android.gcm.GCMBaseIntentService;

public class GCMIntentService extends GCMBaseIntentService {

	private static final String TAG = "GCMIntentService";

	public GCMIntentService() {
		super(SENDER_ID);
	}

	/**
	 * Method called on device registered
	 **/
	@Override
	protected void onRegistered(Context context, String registrationId) {
		Log.i(TAG, "Device registered: regId = " + registrationId);
		displayMessage(context, getString(R.string.gcm_registered));
	}

	/**
	 * Method called on device un registred
	 * */
	@Override
	protected void onUnregistered(Context context, String registrationId) {
		Log.i(TAG, "Device unregistered");
		displayMessage(context, getString(R.string.gcm_unregistered));
	}

	/**
	 * Method called on Receiving a new message
	 * */
	@Override
	protected void onMessage(Context context, Intent intent) {
		Log.i(TAG, "Received message");
		Log.i(TAG, "intent: " + intent.toString());
		try{
			String message = intent.getExtras().getString("message");
			JSONObject challengeObj = new JSONObject(message);
			Log.i(TAG, "message: " + (message == null ? "NOTHING" : message));
			
			int challengeId = challengeObj.getInt("challenge");
			
			String url = RESTCaller.getChallengeInstanceCall(BluetoothAdapter.getDefaultAdapter().getAddress(), challengeId);
			RESTCaller blah = new RESTCaller();
			JSONObject returnedJSONObject = blah.execute(url);
			
			if(!(returnedJSONObject.getBoolean("success"))){
				return;
			}
			
			String challengeString = returnedJSONObject.getString("body");
	
			displayMessage(context, "New Challenge!");
			// notifies user
			generateNotification(context, "New Challenge!", challengeString);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

	/**
	 * Method called on receiving a deleted message
	 * */
	@Override
	protected void onDeletedMessages(Context context, int total) {
		/*
		 * 
		 * Log.i(TAG, "Received deleted messages notification");
		 * String message = getString(R.string.gcm_deleted, total);
		 * displayMessage(context, message);
		 * // notifies user
		 * generateNotification(context, message);*/
	}

	/**
	 * Method called on Error
	 * */
	@Override
	public void onError(Context context, String errorId) {
		Log.i(TAG, "Received error: " + errorId);
		displayMessage(context, getString(R.string.gcm_error, errorId));
	}

	@Override
	protected boolean onRecoverableError(Context context, String errorId) {
		// log message
		Log.i(TAG, "Received recoverable error: " + errorId);
		displayMessage(context,
				getString(R.string.gcm_recoverable_error, errorId));
		return super.onRecoverableError(context, errorId);
	}

	/**
	 * Issues a notification to inform the user that server has sent a message.
	 */
	private static void generateNotification(Context context, String message, String challengejson) {
		int icon = R.drawable.icon;
		long when = System.currentTimeMillis();
		NotificationManager notificationManager = (NotificationManager) context
				.getSystemService(Context.NOTIFICATION_SERVICE);
		Notification notification = new Notification(icon, message, when);

		String title = context.getString(R.string.app_name);

		Intent notificationIntent = new Intent(context,
				ChallengeAcceptanceActivity.class);
		
		notificationIntent.putExtra("challenge_json", challengejson);
		
		// set intent so it does not start a new activity
		notificationIntent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP
				| Intent.FLAG_ACTIVITY_SINGLE_TOP);
		PendingIntent intent = PendingIntent.getActivity(context, 0,
				notificationIntent, 0);
		notification.setLatestEventInfo(context, title, message, intent);
		notification.flags |= Notification.FLAG_AUTO_CANCEL;

		// Play default notification sound
		notification.defaults |= Notification.DEFAULT_SOUND;

		// Vibrate if vibrate is enabled
		notification.defaults |= Notification.DEFAULT_VIBRATE;
		notificationManager.notify(0, notification);

	}

}
